(*
    FCE
    Copyright (c) 1996 Alessandro Scotti
    http://www.ascotti.org/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*)
{$A-}
unit Chord;

interface

uses
  SysUtils, WinTypes, WinProcs, Classes, Graphics, Controls,
  StdCtrls, ExtCtrls;

const
  MAX_FRET = 6;
  FRET_HEIGHT = 7;
  MASK : array[ 1..8 ] of byte = (1,2,4,8,16,32,64,128);
type
  TChord = record
    Name : string[12]; (* Name of chord *)
    Fret : byte;       (* Initial fret *)
    Data : array[ 1..MAX_FRET ] of byte;
    Ring : byte;       (* Unfretted strings *)
    Skip : byte;       (* Unplayed strings *)
  end;
  PChord = ^TChord;

function ChordToBitmap( P: PChord; ShowName: boolean ): TBitmap;

implementation

const
  FRET_NAME : array[ 1..24 ] of string[8] = (
    'I', 'II', 'III', 'IV', 'V', 'VI',
    'VII', 'VIII', 'IX', 'X', 'XI', 'XII',
    'XIII', 'XIV', 'XV', 'XVI', 'XVII', 'XVIII',
    'XIX', 'XX', 'XXI', 'XXII', 'XXIII', 'XXIV'
  );
  FRET_FONT_HEIGHT = 14;

(*
    Converts a chord into a monochrome bitmap.
*)
function ChordToBitmap( P: PChord; ShowName: boolean ): TBitmap;
var
  I, J, W, H, TH, TW: integer;
  B: TBitmap;
  X: array[ 1..6 ] of byte;
  L, T: integer;
  FW: integer;
begin
  (* Count number of frets to display *)
  I := MAX_FRET;
  while( P^.Data[I] = 0 ) do begin
    Dec( I );
    if( I <= 4 ) then
      break;
  end;
  (* Create the bitmap *)
  B := TBitmap.Create;
  with B.Canvas do begin
    Font.Name := 'Courier New';
    FW := 0;
    if( P^.Fret > 1 ) then begin
      Font.Height := FRET_FONT_HEIGHT;
      FW := TextWidth( FRET_NAME[P^.Fret] ) + 2;
    end;
    Font.Height := 18;
  end;
  (* Compute the bitmap size *)
  if( ShowName ) then begin
    TH := B.Canvas.TextHeight( P^.Name );
    TW := B.Canvas.TextWidth( P^.Name );
  end
  else begin
    TW := 0;
    TH := -1;
  end;
  W := 35 + FW;
  L := FW;
  if( TW > W ) then begin
    L := (TW-W) div 2;
    W := TW;
  end;
  H := FRET_HEIGHT*I + 1;
  Inc( H, TH+1 );
  T := TH+1;
  if( P^.Ring <> 0 )or( P^.Skip <> 0 ) then begin
    Inc( H, 6 );
    Inc( T, 6 );
  end;
  (* Compute the X coordinate of each string *)
  for I:=1 to 6 do
    X[I] := 36 - I*6 + L;
  (* Size the bitmap *)
  B.Monochrome := TRUE;
  B.Width := W;
  B.Height := H;
  (* Draw the grid *)
  B.Canvas.Brush.Color := clWhite;
  B.Canvas.Pen.Color := clWhite;
  B.Canvas.Rectangle( 0,0,W,H );
  B.Canvas.Pen.Color := clBlack;
  B.Canvas.Pen.Width := 1;
  for I:=0 to 5 do begin
    B.Canvas.MoveTo( L+2+I*6, T );
    B.Canvas.LineTo( L+2+I*6, T+H );
  end;
  I := T;
  while( I <= H ) do begin
    B.Canvas.MoveTo( L+2, I );
    B.Canvas.LineTo( X[1]+2, I );
    Inc( I, FRET_HEIGHT );
  end;
  (* Draw chord name *)
  if( ShowName ) then
    B.Canvas.TextOut( (W-TW) div 2, 0, P^.Name );
  (* Draw the fret *)
  if( FW > 0 ) then begin
    B.Canvas.Font.Height := FRET_FONT_HEIGHT;
    B.Canvas.TextOut( 0, T-4, FRET_NAME[P^.Fret] );
  end;
  (* Draw skip and ring information *)
  B.Canvas.Brush.Style := bsClear;
  T := TH+1;
  if( P^.Ring <> 0 )or( P^.Skip <> 0 ) then begin
    for I:=1 to 6 do begin
      if( (P^.Skip and MASK[I]) <> 0 ) then begin
        B.Canvas.MoveTo( X[I], T );
        B.Canvas.LineTo( X[I]+5, T+5 );
        B.Canvas.MoveTo( X[I]+4, T );
        B.Canvas.LineTo( X[I]-1, T+5 );
      end;
      if( (P^.Ring and MASK[I]) <> 0 ) then begin
        B.Canvas.Ellipse( X[I], T, X[I]+5, T+5 );
      end;
    end;
    Inc( T, 6 );
  end;
  (* Draw circles *)
  B.Canvas.Brush.Style := bsSolid;
  B.Canvas.Brush.Color := clBlack;
  for J:=1 to MAX_FRET do begin
    for I:=1 to 6 do if( (P^.Data[J] and MASK[I]) <> 0 ) then begin
      B.Canvas.Ellipse( X[I], T+2, X[I]+5, T+7 );
    end;
    Inc( T, FRET_HEIGHT );
  end;
  (* Return *)
  ChordToBitmap := B;
end;

end.
