(*
    FCE
    Copyright (c) 1996 Alessandro Scotti
    http://www.ascotti.org/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*)
unit Main;

interface

uses
  About,
  Chord,
  ChordOpt,
  CopyCmd,
  Database,
  Edit,
  Find,
  Clipbrd,
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, ExtCtrls, Buttons, Menus;

type
  TfrmMain = class(TForm)
    lbSelect: TListBox;
    btnCopy: TBitBtn;
    btnExit: TBitBtn;
    btnDelete: TBitBtn;
    btnNew: TBitBtn;
    btnEdit: TBitBtn;
    pboxPreview: TPaintBox;
    Panel1: TPanel;
    optHideName: TCheckBox;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    mnuOpen: TMenuItem;
    N1: TMenuItem;
    mnuExit: TMenuItem;
    Options1: TMenuItem;
    mnuSetupChord: TMenuItem;
    mnuSetupFastCopy: TMenuItem;
    Help1: TMenuItem;
    mnuContents: TMenuItem;
    mnuHowToUseHelp: TMenuItem;
    N3: TMenuItem;
    mnuAbout: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnCopyClick(Sender: TObject);
    procedure btnNewClick(Sender: TObject);
    procedure pboxPreviewPaint(Sender: TObject);
    procedure lbSelectClick(Sender: TObject);
    procedure btnEditClick(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
    procedure optHideNameClick(Sender: TObject);
    procedure mnuSetupFastCopyClick(Sender: TObject);
    procedure mnuAboutClick(Sender: TObject);
    procedure mnuExitClick(Sender: TObject);
    procedure mnuSetupChordClick(Sender: TObject);
  private
    { Private declarations }
    hwndTarget : HWnd;
    iCopyCmd   : integer;
    procedure FastCopy;
    function GetCurChord: PChord;
  public
    { Public declarations }
    ClipBoard: TClipBoard;
  end;

procedure Keybd_Event;

var
  frmMain: TfrmMain;

implementation

{$R *.DFM}

function TfrmMain.GetCurChord: PChord;
var
  I: integer;
begin
  I := lbSelect.ItemIndex;
  if( I < 0 ) then
    Result := nil
  else
    Result := PChord(lbSelect.Items.Objects[I]);
end;

procedure TfrmMain.FormCreate(Sender: TObject);
var
  I: integer;
begin
  ClipBoard := TClipBoard.Create;
  hwndTarget := 0;
  (* *)
  LoadChords( lbSelect );
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  SaveChords( lbSelect );
  ClipBoard.Free;
end;

procedure TfrmMain.btnCopyClick(Sender: TObject);
var
  P: PChord;
  B: TBitmap;
begin
  P := GetCurChord;
  if( P = nil ) then
    Exit;
  B := ChordToBitmap( P, not optHideName.Checked );
  Clipboard.Assign( B );
  B.Free;
  FastCopy;
end;

procedure TfrmMain.btnNewClick(Sender: TObject);
var
  I: integer;
  P: PChord;
begin
  with frmEdit.Chord do begin
    Name := '';
    Ring := 0;
    Skip := 0;
    Fret := 1;
    for I:=1 to MAX_FRET do Data[I] := 0;
  end;
  I := frmEdit.ShowModal;
  if( I = mrOk ) then begin
    P := NewChord;
    P^ := frmEdit.Chord;
    lbSelect.Items.AddObject( P^.Name, TObject(P) );
  end;
end;

procedure TfrmMain.pboxPreviewPaint(Sender: TObject);
var
  B: TBitmap;
  X, Y: integer;
  RS, RD: TRect;
  P: PChord;
begin
  with pboxPreview do begin
    Canvas.Brush.Color := clWhite;
    Canvas.Rectangle( 0, 0, Width, Height );
    P := GetCurChord;
    if( P = nil ) then
      Exit;
    B := ChordToBitmap( P, not optHideName.Checked );
    RS := Rect( 0, 0, B.Width, B.Height );
    X := (Width-B.Width) div 2;
    Y := (Height-B.Height) div 2;
    RD := Rect( X, Y, X+B.Width, Y+B.Height );
    Canvas.CopyRect( RD, B.Canvas, RS );
    B.Free;
  end;
end;

procedure TfrmMain.lbSelectClick(Sender: TObject);
begin
  pboxPreview.Refresh;
end;

procedure TfrmMain.btnEditClick(Sender: TObject);
var
  P: PChord;
  I: integer;
begin
  P := GetCurChord;
  if( P = nil ) then
    Exit;
  frmEdit.Chord := P^;
  I := frmEdit.ShowModal;
  if( I = mrOk ) then begin
    if( P^.Name <> frmEdit.Chord.Name ) then begin
      lbSelect.Items.Delete( lbSelect.ItemIndex );
      P := NewChord;
      P^ := frmEdit.Chord;
      lbSelect.Items.AddObject( P^.Name, TObject(P) );
    end
    else begin
      P^ := frmEdit.Chord;
    end;
    pboxPreview.Refresh;
  end;
end;

procedure TfrmMain.btnDeleteClick(Sender: TObject);
var
  I: integer;
begin
  I := lbSelect.ItemIndex;
  if( I < 0 ) then
    Exit;
  lbSelect.Items.Delete( I );
  pboxPreview.Refresh;
end;

procedure TfrmMain.BitBtn1Click(Sender: TObject);
var
  iRes: integer;
begin
  (*frmAbout.ShowModal;*)
end;

procedure TfrmMain.optHideNameClick(Sender: TObject);
begin
  pboxPreview.Refresh;
end;

const
  KUP       = TRUE;
  KDOWN     = FALSE;

procedure Keybd_Event; external 'USER.EXE';

procedure PostVirtualKey( VirtualKey: byte; Up: boolean );
var
  RegAX, RegBX: word;
begin
  RegBX := MapVirtualKey( VirtualKey, 0 );
  RegAX := VirtualKey;
  if( Up ) then Inc( RegAX, $8000 );
  asm
    mov ax, [RegAX]
    mov bx, [RegBX]
  end;
  Keybd_Event;
end;

procedure TfrmMain.FastCopy;
begin
  if( hwndTarget = 0 ) then
    Exit;
  SetActiveWindow( hwndTarget );
  Application.ProcessMessages;
  case iCopyCmd of
    1: begin
      PostVirtualKey( VK_CONTROL, KDOWN );
      PostVirtualKey( VK_INSERT, KDOWN );
      PostVirtualKey( VK_INSERT, KUP );
      PostVirtualKey( VK_CONTROL, KUP );
      end;
    2: begin
      PostVirtualKey( VK_CONTROL, KDOWN );
      PostVirtualKey( byte('V'), KDOWN );
      PostVirtualKey( byte('V'), KUP );
      PostVirtualKey( VK_CONTROL, KUP );
      end;
    3: begin
      PostVirtualKey( VK_CONTROL, KDOWN );
      PostVirtualKey( byte('I'), KDOWN );
      PostVirtualKey( byte('I'), KUP );
      PostVirtualKey( VK_CONTROL, KUP );
      end;
  end;
end;

procedure TfrmMain.mnuSetupFastCopyClick(Sender: TObject);
var
  iRes: integer;
  boChecked: boolean;
begin
  boChecked := not mnuSetupFastCopy.Checked;
  mnuSetupFastCopy.Checked := boChecked;
  hwndTarget := 0;
  if( not boChecked ) then
    Exit;
  iRes := frmFindWnd.ShowModal;
  if( iRes = mrOk ) then begin
    iRes := frmCopyCmd.ShowModal;
    if( iRes = mrOk ) then begin
      hwndTarget := frmFindWnd.hwndTarget;
      iCopyCmd := frmCopyCmd.iCopyCmd;
    end;
  end;
end;

procedure TfrmMain.mnuAboutClick(Sender: TObject);
begin
  frmAbout.ShowModal;
end;

procedure TfrmMain.mnuExitClick(Sender: TObject);
begin
  Application.Terminate;
end;

procedure TfrmMain.mnuSetupChordClick(Sender: TObject);
begin
  frmChordOptions.ShowModal;
end;

end.
